//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#pragma once

#include "CDPChangeNotifications.h"
#include "CDPString.h"

#include "cdphlMetaData.h"

#include <vector>

namespace cdphl {


//==================================================================================================
//
// 
//
//==================================================================================================
class ChangeNotifications : public CDPChangeNotifications
{
public:
    ChangeNotifications();
    ~ChangeNotifications();
    
    virtual void            notifyResultChanged(cdp_int_t resultId) override;
    virtual CDPResultInfo*  notifyResultAdded(CDPResultType resultType, cdp_int_t resultId, const CDPString& name, CDPResultMappingType mapping) override;
    virtual void            notifyResultDeleted(cdp_int_t resultId) override;
    
    virtual void            notifyRigidBodyTransformationsChanged() override;
    
    virtual void            notifyStateMetaDataChanged(cdp_int_t stateId, const CDPString& name, cdp_double_t referenceValue) override;
    virtual CDPStateInfo*   notifyStateAdded(cdp_int_t stateId, const CDPString& name, cdp_double_t referenceValue) override;
    virtual void            notifyStateDeleted(cdp_int_t stateId) override;
    
    virtual void            notifyGeometryChanged(cdp_size_t geometryIndex) override;
    
    bool                    resultIdExists(cdp_int_t resultId) const;
    bool                    stateIdExists(cdp_int_t stateId) const;

public:
    std::vector<cdp_int_t>          resultChangedIdArr;
    std::vector<ResultInfo*>        resultAddedInfoArr;
    std::vector<cdp_int_t>          resultDeletedIdArr;
    
    bool                            rigidBodyTransformationsChanged;

    std::vector<StateInfo*>         stateMetaDataChangedInfoArr;
    std::vector<StateInfo*>         stateAddedInfoArr;
    std::vector<cdp_int_t>          stateDeletedIdArr;

    std::vector<cdp_size_t>         geometryChangedIndexArr;
};

}  
