//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#include "cdphlDataProviderFactory.h"
#include "cdphlAssert.h"
#include "CDPString.h"

#include "CDPDataProviderFactory.h"

namespace cdphl {


//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
DataProviderFactory::DataProviderFactory(int sequenceNumber, const char* providerIdString, CDPDataProviderFactory* providerFactoryFromPlugin, CDP_DestroyProviderFactoryFunc destroyProviderFactoryFunc)
 :  m_sequenceNumber(sequenceNumber),
    m_idString(providerIdString),
    m_providerFactoryFromPlugin(providerFactoryFromPlugin),
    m_destroyProviderFactoryFunc(destroyProviderFactoryFunc)
{
    CDPHL_ASSERT(m_providerFactoryFromPlugin);
    CDPHL_ASSERT(m_destroyProviderFactoryFunc);
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
DataProviderFactory::~DataProviderFactory()
{
    CDPHL_ASSERT(m_providerFactoryFromPlugin);
    CDPHL_ASSERT(m_destroyProviderFactoryFunc);

    m_destroyProviderFactoryFunc(m_providerFactoryFromPlugin);
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
int DataProviderFactory::sequenceNumber() const
{
    return m_sequenceNumber;
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
std::string DataProviderFactory::providerId() const
{
    return m_idString;
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool DataProviderFactory::getProviderInfo(CDPDataProviderInfo* info)
{
    CDPHL_ASSERT(m_providerFactoryFromPlugin);
    return m_providerFactoryFromPlugin->getProviderInfo(info);
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool DataProviderFactory::isSupportedByProvider(const CDPString& modelKey)
{
    CDPHL_ASSERT(m_providerFactoryFromPlugin);
    return m_providerFactoryFromPlugin->isSupportedByProvider(modelKey);
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
CDPDataProvider* DataProviderFactory::createProviderInstance()
{
    CDPHL_ASSERT(m_providerFactoryFromPlugin);

    return m_providerFactoryFromPlugin->createProviderInstance();
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void DataProviderFactory::destroyProviderInstance(CDPDataProvider* instance)
{
    CDPHL_ASSERT(m_providerFactoryFromPlugin);

    if (instance)
    {
        m_providerFactoryFromPlugin->destroyProviderInstance(instance);
    }
}

} // cdphl

