//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#pragma once

#include "cdphlBase.h"

#include "CDPPlugin.h"
#include "CDPDataProviderFactory.h"
#include "CDPString.h"

#include <vector>

namespace cdphl {

//==================================================================================================
//
//
//
//==================================================================================================
enum DataType
{
    DOUBLE,
    BOOL,
    STRING
};

//==================================================================================================
//
//
//
//==================================================================================================
struct InitOption
{
    CDPString   key;
    CDPString   description;
    DataType    dataType;
    
    double      doubleValue;
    bool        boolValue;
    CDPString   stringValue;
};

//==================================================================================================
//
//
//
//==================================================================================================
class DataProviderInfo : public CDPDataProviderInfo
{
public:
    virtual void        setFileDescription(const CDPString& description) override;
    virtual void        setFileMask(const CDPString& mask) override;
    
    virtual void        publishInitOptionBool(const CDPString& optionKey, const CDPString& optionDescription, bool defaultValue) override;
    virtual void        publishInitOptionDouble(const CDPString& optionKey, const CDPString& optionDescription, cdp_double_t defaultValue) override;
    virtual void        publishInitOptionString(const CDPString& optionKey, const CDPString& optionDescription, const CDPString& defaultValue) override;
    
    const CDPString&    fileDescription() const;
    const CDPString&    fileMask() const;
    
    size_t              initOptionCount() const;
    const InitOption&   initOption(size_t index) const;

private:
    CDPString               m_fileDescription;
    CDPString               m_fileMask;
    
    std::vector<InitOption> m_initOptions;
};

}
