//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#include "cdphlDynamicLibrary.h"

#ifdef WIN32
  #include <Windows.h>
#else
  #include <dlfcn.h>
#endif

#include <sstream>
#include <iostream>

namespace cdphl {



//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
DynamicLibrary::DynamicLibrary(void * handle) : m_handle(handle)
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
DynamicLibrary::~DynamicLibrary()
{
    if (m_handle)
    {
#ifndef WIN32
        ::dlclose(m_handle);
#else
        ::FreeLibrary((HMODULE)m_handle);
#endif
    }
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
DynamicLibrary * DynamicLibrary::load(const std::string& name, std::string* errorString)
{
    if (name.empty())
    {
        *errorString = "Empty path.";
        return NULL;
    }

    void* handle = NULL;

#ifdef WIN32

    handle = ::LoadLibraryA(name.c_str());
    if (handle == NULL)
    {
        DWORD errorCode = ::GetLastError();
        std::stringstream ss;
        ss << std::string("LoadLibrary(") << name << std::string(") Failed. errorCode: ") << errorCode;
        *errorString = ss.str();
        return NULL;
    }

#else

    handle = ::dlopen(name.c_str(), RTLD_NOW);
    if (!handle)
    {
        std::string dlErrorString;
        const char* zErrorString = ::dlerror();
        if (zErrorString)
        {
            dlErrorString = zErrorString;
        }

        *errorString += "Failed to load \"" + name + '"';
        if (dlErrorString.size())
        {
            *errorString += ": " + dlErrorString;
        }
      
        return NULL;
    }

#endif

  return new DynamicLibrary(handle);
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void * DynamicLibrary::getSymbol(const std::string & symbol)
{
    if (!m_handle)
    {
        return NULL;
    }
  
#ifdef WIN32
    return ::GetProcAddress((HMODULE)m_handle, symbol.c_str());
#else
    return ::dlsym(m_handle, symbol.c_str());
#endif
}


} // cdphl
