//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#pragma once

#include "cdphlBase.h"

#include "CDPElementGroup.h"

#include <vector>

namespace cdphl {


//==================================================================================================
//
// 
//
//==================================================================================================
struct ElementSubGroupData
{
    CDPElementType                  elementType;
    cdp_size_t                      elementCount;
    std::vector<CDPElementType>     elementTypeArr;
    std::vector<cdp_int_t>          elementNodeArr;
    std::vector<cdp_int_t>          elementIdArr;
    
    // Derived data for all element types
    std::vector<cdp_int_t>          elementStartArr;
    
    size_t  computeDerivedData(const std::vector<cdp_ubyte_t>& polyhedronNumNodesPerElement, size_t polyhedronElementIndexOffset);
    int     numElementNodes(size_t elementIndex) const;
};


//==================================================================================================
//
// 
//
//==================================================================================================
class ElementGroup : public CDPElementGroup
{
public:
    ElementGroup(cdp_size_t nodeGroupIndex);
    virtual ~ElementGroup();
    
    virtual void        addSingleTypeElements(CDPElementType elementType, cdp_size_t elementCount, const cdp_int_t* elementNodeArr, cdp_size_t elementNodeCount, const cdp_int_t* elementIdArr) override;
    virtual void        addMultipleTypeElements(const CDPElementType* elementTypeArr, cdp_size_t elementCount, const cdp_int_t* elementNodeArr, cdp_size_t elementNodeCount, const cdp_int_t* elementIdArr) override;
    virtual void        addPolyhedronData(const cdp_ubyte_t* numNodesPerPolyhedronArr, const cdp_ubyte_t* numFacesPerPolyhedronArr, cdp_size_t polyhedronCount, const cdp_ubyte_t* numNodesPerFaceArr, cdp_size_t totalFaceCount, const cdp_int_t* faceNodeIndicesArr, cdp_size_t faceNodeIndicesCount) override;

    virtual void        setElementPartIds(const cdp_int_t* partIdArr, cdp_size_t elementCount) override;

    virtual void        setPrescribedElementOrdering(const cdp_int_t* elementTargetIndices, cdp_size_t elementCount) override;

    void                computeDerivedData();

    size_t                              nodeGroupIndex() const;
                                        
    size_t                              subGroupDataCount() const;
    const ElementSubGroupData&          subGroupData(size_t index) const;
                                        
    const std::vector<cdp_ubyte_t>&     polyhedronFaceCountArr() const;
    const std::vector<cdp_ubyte_t>&     polyhedronFaceNodeCountArr() const;
    const std::vector<cdp_int_t>&       polyhedronFaceNodeIndicesArr() const;
                                        
    const std::vector<cdp_int_t>&       elementPartIdArr() const;
    const std::vector<cdp_int_t>&       prescribedElementIndices() const;
                                        
    size_t                              elementCount() const;
                                        
    static size_t                       fixedNodeCountForElement(CDPElementType elementType);
                                      
private:
    cdp_size_t                          m_nodeGroupIndex;
    std::vector<ElementSubGroupData>    m_subGroupDataArr;
    std::vector<cdp_int_t>              m_elementPartIdArr;
    std::vector<cdp_int_t>              m_prescribedElementIndices;

    std::vector<cdp_ubyte_t>            m_polyhedronNumNodesPerElement;
    std::vector<cdp_int_t>              m_polyhedronFaceNodeIndicesArr;
    std::vector<cdp_ubyte_t>            m_polyhedronFaceCountArr;
    std::vector<cdp_ubyte_t>            m_polyhedronFaceNodeCountArr;
};

    
}  // cdphl

