//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#include "cdphlGeometry.h"
#include "cdphlNodeGroup.h"
#include "cdphlElementGroup.h"
#include "cdphlAssert.h"

#include <set>

namespace cdphl {


//==================================================================================================
//
//
//
//==================================================================================================
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
Geometry::~Geometry()
{
    for (size_t i = 0; i < m_nodeGroupArr.size(); ++i)
    {
        delete m_nodeGroupArr[i];
    }

    for (size_t i = 0; i < m_elementGroupArr.size(); ++i)
    {
        delete m_elementGroupArr[i];
    }
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
CDPNodeGroup* Geometry::addNodeGroup()
{
    size_t groupIdx = m_nodeGroupArr.size();
    NodeGroup* group = new NodeGroup(groupIdx);
    
    m_nodeGroupArr.push_back(group);
        
    return group;
}
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
CDPElementGroup* Geometry::addElementGroup(cdp_size_t nodeGroupIdx)
{
    CDPHL_ASSERT(nodeGroupIdx != CDP_UNDEFINED_SIZE_T);
    
    ElementGroup* group = new ElementGroup(nodeGroupIdx);
    m_elementGroupArr.push_back(group);
    
    return group;
}
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
size_t Geometry::nodeGroupCount() const
{
    return m_nodeGroupArr.size();
}
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
const NodeGroup& Geometry::nodeGroup(size_t index) const
{
    return *m_nodeGroupArr[index];
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
size_t Geometry::elementGroupCount() const
{
    return m_elementGroupArr.size();
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
const ElementGroup& Geometry::elementGroup(size_t index) const
{
    return *m_elementGroupArr[index];
}
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void Geometry::computeDerivedData()
{
    for (size_t i = 0; i < m_elementGroupArr.size(); ++i)
    {
        CDPHL_ASSERT(m_elementGroupArr[i]);
        m_elementGroupArr[i]->computeDerivedData();
    }
}

    
//==================================================================================================
//
//
//
//==================================================================================================
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
GeometrySpec::GeometrySpec()
:   m_geometryIndex(0),
    m_stateId(-1)
{
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
GeometrySpec::GeometrySpec(cdp_size_t geometryIndex, cdp_int_t stateId)
:   m_geometryIndex(geometryIndex),
    m_stateId(stateId)
{
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void GeometrySpec::set(cdp_size_t geometryIndex, cdp_int_t stateId)
{
    m_geometryIndex = geometryIndex;
    m_stateId = stateId;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cdp_size_t GeometrySpec::geometryIndex() const
{
    return m_geometryIndex;
}
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cdp_int_t GeometrySpec::stateId() const
{
    return m_stateId;
}


}  // cdphl
