//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#pragma once

#include "cdphlBase.h"
#include "CDPGeometry.h"

#include <vector>

namespace cdphl {

class NodeGroup;
class ElementGroup;


//==================================================================================================
//
// 
//
//==================================================================================================
class Geometry : public CDPGeometry 
{
public:
    virtual ~Geometry();
    
    virtual CDPNodeGroup*       addNodeGroup() override;
    virtual CDPElementGroup*    addElementGroup(cdp_size_t nodeGroupIdx) override;

    size_t                      nodeGroupCount() const;
    const NodeGroup&            nodeGroup(size_t index) const;
    
    size_t                      elementGroupCount() const;
    const ElementGroup&         elementGroup(size_t index) const;

    void                        computeDerivedData();

private:
    std::vector<NodeGroup*>     m_nodeGroupArr;
    std::vector<ElementGroup*>  m_elementGroupArr;
};


//==================================================================================================
//
// 
//
//==================================================================================================
class GeometrySpec : public CDPGeometrySpec
{
public:
    GeometrySpec();
    GeometrySpec(cdp_size_t geometryIndex, cdp_int_t stateId);

    virtual cdp_size_t  geometryIndex() const override;
    virtual cdp_int_t   stateId() const override;

    void                set(cdp_size_t geometryIndex, cdp_int_t stateId);

private:
    cdp_size_t  m_geometryIndex;
    cdp_int_t   m_stateId;
};


}  // cdphl

