//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#include "cdphlInitOptions.h"
#include <string>
#include <limits>

namespace cdphl {

//==================================================================================================
//
//
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
InitOptions::InitOptions()
{
}


//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
InitOptions::InitOptions(const InitOptions& other)
{
    *this = other;
}


//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
InitOptions& InitOptions::operator=(const InitOptions& rhs)
{
    m_boolKeyValueMap = rhs.m_boolKeyValueMap;
    m_doubleKeyValueMap = rhs.m_doubleKeyValueMap;
    m_stringKeyValueMap = rhs.m_stringKeyValueMap;
    
    return *this;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool InitOptions::hasKey(const CDPString& optionKey) const
{
    std::string key = optionKey.utf8();
    
    return (m_boolKeyValueMap.find(key) != m_boolKeyValueMap.end() ||
            m_doubleKeyValueMap.find(key) != m_doubleKeyValueMap.end() ||
            m_stringKeyValueMap.find(key) != m_stringKeyValueMap.end());
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cdp_double_t InitOptions::doubleValue(const CDPString& optionKey) const
{
    std::string key = optionKey.utf8();
    
    std::map<std::string, double>::const_iterator it = m_doubleKeyValueMap.find(key);
    if (it != m_doubleKeyValueMap.end())
    {
        return it->second;
    }
    
    return std::numeric_limits<double>::infinity();
}


//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool InitOptions::boolValue(const CDPString& optionKey) const
{
    std::string key = optionKey.utf8();
    
    std::map<std::string, bool>::const_iterator it = m_boolKeyValueMap.find(key);
    if (it != m_boolKeyValueMap.end())
    {
        return it->second;
    }
    
    return false;
}


//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
const CDPString& InitOptions::stringValue(const CDPString& optionKey) const
{
    std::string key = optionKey.utf8();
    
    std::map<std::string, CDPString>::const_iterator it = m_stringKeyValueMap.find(key);
    if (it != m_stringKeyValueMap.end())
    {
        return it->second;
    }
    
    return CDPString::emptyString();
}


//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void InitOptions::setDoubleValue(const CDPString& optionKey, double value)
{
    std::string key = optionKey.utf8();
    m_doubleKeyValueMap[key] = value;
}


//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void InitOptions::setBoolValue(const CDPString& optionKey, bool value)
{
    std::string key = optionKey.utf8();
    m_boolKeyValueMap[key] = value;
}


//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void InitOptions::setStringValue(const CDPString& optionKey, const CDPString& value)
{
    std::string key = optionKey.utf8();
    m_stringKeyValueMap[key] = value;
}

} // cdphl
