//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#pragma once

#include "CDPPlugin.h"

#include <string>
#include <map>
#include <vector>

namespace cdphl {

class DataProviderFactory;
class DynamicLibrary;
class Logger;



//==================================================================================================
//
// 
//
//==================================================================================================
class PluginManager
{
public:
    static PluginManager*       instance();
    void                        shutdown();

    void                        setLogger(Logger* logger);

    bool                        loadPlugin(const std::string& pluginBaseFilename);
    //bool                        loadAllPlugins(const std::string& directory);

    bool                        initializeStaticPlugin(CDP_PluginInitFunc pluginInitFunc);

    size_t                      providerCount() const;
    DataProviderFactory*        providerFactory(size_t index) const;

private:
    PluginManager();
    ~PluginManager();

    bool            doInitializationOfPlugin(CDP_PluginInitFunc pluginInitFunc, DynamicLibrary* dynLib, std::string fullPluginFilename);
    bool            hasProviderWithId(const char* providerIdString) const;

    int             handleProviderRegistration(const char* providerIdString, const CDP_RegisterProviderParams* params);
    static int      staticHandleProviderRegistration(const char* providerIdString, const CDP_RegisterProviderParams* params);

    void            handleSetPluginInfo(const char* key, const char* value);
    static void     staticHandleSetPluginInfo(const char* key, const char* value);

    static void     staticLogFunction(int level, const char* message);

private:
    struct PluginEntry
    {
        std::string                         fullPluginFilename;
        DynamicLibrary*                     dynLib;
        CDP_PluginExitFunc                  exitFunc;
        std::map<std::string, std::string>  pluginInfoKeyValues;
    };

    Logger*                             m_logger;
    CDP_FrameworkServices               m_frameworkServices;

    PluginEntry*                        m_pluginBeingInitialized;
    std::vector<PluginEntry>            m_pluginEntryArr;
    int                                 m_lastAssignedSequenceNumber;
    std::vector<DataProviderFactory*>   m_providerFactoryArr;
};

}

