//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#pragma once

#include "CDPBaseTypes.h"
#include "CDPInitOptions.h"
#include "CDPResult.h"

class CDPMetaData;
class CDPGeometrySpec;
class CDPGeometry;
class CDPVisibilityResult;
class CDPSet;
class CDPTransformations;
class CDPChangeNotifications;

class CDPDataProvider
{
public:
    virtual ~CDPDataProvider() {};

    virtual bool init(const CDPString& modelKey, const CDPInitOptions& initOptions) = 0;

    virtual void close() = 0;
    
    virtual bool getMetaData(CDPMetaData* metaData) = 0;
    virtual bool getGeometry(const CDPGeometrySpec& spec, CDPGeometry* geometry) = 0;
    
    virtual bool getSet(cdp_int_t setId, CDPSet* set) = 0;

    virtual bool getResult(const CDPResultSpec& spec, CDPResult* result) = 0;
    virtual bool getRigidBodyTransformations(cdp_size_t geometryIndex, cdp_int_t stateId, CDPTransformations* transformations) = 0;
    virtual bool getElementVisibilityResult(cdp_size_t geometryIndex, cdp_int_t stateId, CDPVisibilityResult* result) = 0;
    
    virtual void pollForChanges(CDPChangeNotifications* changeNotifications) = 0;
};
