//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#include "CDPDataProviderFactory.h"
#include "CDPBaseTypes.h"
#include "CDPString.h"
#include <cstddef>

//==================================================================================================
///
/// \class CDPDataProviderInfo
///
/// This class contains information about the data provider
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Specify the desciption of the files/content the provider supports.
///
/// Example: "ANSYS Binary Result File"
//--------------------------------------------------------------------------------------------------
void CDPDataProviderInfo::setFileDescription(const CDPString& description)
{
}

//--------------------------------------------------------------------------------------------------
/// Specify (if applicable) the file mask of the files supported by this provider
///
/// Example: *.rst;*.rth;*.rfl" 
//--------------------------------------------------------------------------------------------------
void CDPDataProviderInfo::setFileMask(const CDPString& mask)
{
}

//--------------------------------------------------------------------------------------------------
/// Publish information about a boolean initialization option for the data provider.
///
/// These settings will be available for the host program as ReaderSettings, and user defined values 
/// will be provided in the CDPInitOptions passed to init() in the DataProvider.
//--------------------------------------------------------------------------------------------------
void CDPDataProviderInfo::publishInitOptionBool(const CDPString& optionKey, const CDPString& optionDescription, bool defaultValue)
{
}

//--------------------------------------------------------------------------------------------------
/// Publish information about a double initialization option for the data provider.
///
/// These settings will be available for the host program as ReaderSettings, and user defined values 
/// will be provided in the CDPInitOptions passed to init() in the DataProvider.
//--------------------------------------------------------------------------------------------------
void CDPDataProviderInfo::publishInitOptionDouble(const CDPString& optionKey, const CDPString& optionDescription, cdp_double_t defaultValue)
{
}

//--------------------------------------------------------------------------------------------------
/// Publish information about a string initialization option for the data provider.
///
/// These settings will be available for the host program as ReaderSettings, and user defined values 
/// will be provided in the CDPInitOptions passed to init() in the DataProvider.
//--------------------------------------------------------------------------------------------------
void CDPDataProviderInfo::publishInitOptionString(const CDPString& optionKey, const CDPString& optionDescription, const CDPString& defaultValue)
{
}


//==================================================================================================
///
/// \class CDPDataProviderFactory
/// 
/// A Data Provider Plugin has two classes. One CDPDataProviderFactory and one CDPDataProvider.
/// The host will instantiate the factory once (usually at startup), and use the factory to 
/// determine if a given modelKey is supported by the plugin or not.
/// 
/// If the modelKey is supported, the host will use the factory to create a CDPDataProvider instance. 
/// Whenever the model/analysis is closed, the CDPDataProvider will be deleted.
///
/// The factory is only created once per process, and usually kept until the app/server closes.
///
/// The factory has two roles: 
///
///   -#  Provide information about the associated data provider and check if a modelKey is supported by the
///       provider.
///   -#  Create and destroy the associated data provider.
//==================================================================================================


//--------------------------------------------------------------------------------------------------
/// Get information about the provider produced by this factory.
/// 
/// Populate the info object with information relevant to your provider.
//--------------------------------------------------------------------------------------------------
bool CDPDataProviderFactory::getProviderInfo(CDPDataProviderInfo* info)
{
    return false;
}

//--------------------------------------------------------------------------------------------------
/// Check if the given modelKey is supported by the data providers produced by this factory.
/// 
/// In this method you need to check if the incoming modelKey points to a resource that your
/// provider can read. 
/// 
/// Return true if the modelKey is supported, false if it is not.
/// 
/// If you return true, the next action from the host will be to create a CDPDataProvider with the 
/// createProviderInstance() method, and then call init on this object with the given modelKey.
//--------------------------------------------------------------------------------------------------
bool CDPDataProviderFactory::isSupportedByProvider(const CDPString& modelKey)
{
    return false;
}

//--------------------------------------------------------------------------------------------------
/// Create and return an instance of the data provider associated with this factory.
/// 
/// Create and return the new instance. The instance is managed by the host, but it will be deleted
/// with the destroyProviderInstance() method to ensure that it is allocated and deleted with the 
/// same heap manager.
//--------------------------------------------------------------------------------------------------
CDPDataProvider* CDPDataProviderFactory::createProviderInstance()
{
    return NULL;
}

//--------------------------------------------------------------------------------------------------
/// Destroy the given data provider.
/// 
/// This method will be called whenever the host is done with the use of the data provider.
/// The provider passed to this method will always have been allocated with createProviderInstance() in
/// the same factory. This is to ensure that it is allocated and deleted with the same heap manager.
//--------------------------------------------------------------------------------------------------
void CDPDataProviderFactory::destroyProviderInstance(CDPDataProvider* instance)
{
}

