//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

/// \file CDPDataProviderTypes.h
/// Global types for Ceetron Data Provider Framework

#pragma once

/// Element types supported by this framework. All element types except for CDP_ELT_POLYHEDRONS have a fixed topology.
enum CDPElementType
{
    CDP_ELT_UNKNOWN,            ///< Unknown element. Useful when having elements not supported by CDP, but you want to keep the ordering and indices of elements (e.g. for results)
    CDP_ELT_BEAMS,              ///< Beam element with 2 nodes
    CDP_ELT_BEAMS_3,            ///< Beam element with 3 nodes
    CDP_ELT_TRIANGLES,          ///< Triangle element with 3 nodes
    CDP_ELT_TRIANGLES_6,        ///< High order triangle element with 6 nodes
    CDP_ELT_QUADS,              ///< Quad element with 4 nodes
    CDP_ELT_QUADS_8,            ///< High order quad element with 8 nodes
    CDP_ELT_QUADS_9,            ///< High order quad element with 9 nodes (including center node)
    CDP_ELT_TETRAHEDRONS,       ///< Tetrahedron element with 4 nodes
    CDP_ELT_TETRAHEDRONS_10,    ///< High order tetrahedron element with 10 nodes
    CDP_ELT_HEXAHEDRONS,        ///< Hexahedron element with 8 nodes
    CDP_ELT_HEXAHEDRONS_20,     ///< High order hexahedron element with 20 nodes
    CDP_ELT_PENTAHEDRONS,       ///< Pentahedron element with 6 nodes
    CDP_ELT_PENTAHEDRONS_15,    ///< High order pentahedron element with 15 nodes
    CDP_ELT_POINTS,             ///< Point element with 1 node
    CDP_ELT_PYRAMIDS,           ///< Pyramid element with 5 nodes
    CDP_ELT_PYRAMIDS_13,        ///< Pyramid element with 13 nodes
    CDP_ELT_POLYHEDRONS         ///< Polyhedron element with any number of surfaces and any number of nodes per surface
};


/// Set type
enum CDPSetType
{
    CDP_SET_ELEMENTS            ///< Element sets
};


/// The result types supported by this framework.
enum CDPResultType
{
    CDP_RESTYPE_SCALAR,         ///< Scalar result. 1 value per item
    CDP_RESTYPE_VECTOR,         ///< Vector result. 3 values per item
    CDP_RESTYPE_DISPLACEMENT,   ///< Displacement result. 3 values per item
    CDP_RESTYPE_STRESS_TENSOR,  ///< Stress tensor result. 6 values per item
    CDP_RESTYPE_STRAIN_TENSOR   ///< Strain tensor result. 6 values per item
};


/// The result mapping types (result positions) supported by this framework.
enum CDPResultMappingType
{
    CDP_PER_NODE,               ///< Per node
    CDP_PER_ELEMENT,            ///< Per element
    CDP_PER_ELEMENT_NODE,       ///< Per element node
    CDP_PER_ELEMENT_SURFACE     ///< Per element surface
};

// FE model analysis types
enum CDPSimulationType
{
    CDP_SIMULATION_TYPE_STRUCTURAL,
    CDP_SIMULATION_TYPE_THERMAL,
    CDP_SIMULATION_TYPE_ELECTRIC,
    CDP_SIMULATION_TYPE_MAGNETIC,
    CDP_SIMULATION_TYPE_FLUID,
    CDP_SIMULATION_TYPE_ACOUSTIC,
    CDP_SIMULATION_TYPE_DIFFUSION,
    CDP_SIMULATION_TYPE_OTHER
};


// FE model solution types
enum CDPSolutionType
{
    CDP_SOLUTION_TYPE_STATIC,
    CDP_SOLUTION_TYPE_STEADY_STATE,
    CDP_SOLUTION_TYPE_VIBRATION,
    CDP_SOLUTION_TYPE_BUCKLING,
    CDP_SOLUTION_TYPE_TRANSIENT,
    CDP_SOLUTION_TYPE_SUPERELEMENT,
    CDP_SOLUTION_TYPE_FREQRESPONSE,
    CDP_SOLUTION_TYPE_COMPLEXEIGEN,
    CDP_SOLUTION_TYPE_QUASISTATIC,
    CDP_SOLUTION_TYPE_OUTPUT,
    CDP_SOLUTION_TYPE_SINGULAR,
    CDP_SOLUTION_TYPE_OTHER
};
