//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#include "CDPGeometry.h"


//==================================================================================================
///
/// \class CDPGeometry
/// 
/// A geometry is defined by a number of element and node groups. The element groups specify how
/// many parts the geometry will end up having in the host app, either one part per group or any
/// number of parts per group. See CDPGeometryInfo for more information about parts.
/// 
/// Each element group can have its own node group or share it with other element groups. The node group
/// to use for an element group is specified in the call to addElementGroup. The normal case is either
/// to have one node group to share between all element groups, or to have one node group per element
/// group.
/// 
/// A geometry must have at least one node and one element group.
///
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Add a node group to this geometry.
/// 
/// Use the returned CDPNodeGroup to populate the group with coordinates and optional ids.
//--------------------------------------------------------------------------------------------------
CDPNodeGroup* CDPGeometry::addNodeGroup()
{
    return NULL;
}

//--------------------------------------------------------------------------------------------------
/// Add an element group to this geometry.
///
/// Use the returned CDPElementGroup to populate the elements of the group.
//--------------------------------------------------------------------------------------------------
CDPElementGroup* CDPGeometry::addElementGroup(cdp_size_t nodeGroupIdx)
{
    return NULL;
}


//==================================================================================================
///
/// \class CDPGeometrySpec
///
/// A identifier of a [geometry](@ref CDPGeometry).
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// The zero based index of the geometry to get. 
/// 
/// For the normal case of only one geometry, this will be 0.
//--------------------------------------------------------------------------------------------------
cdp_size_t CDPGeometrySpec::geometryIndex() const
{
    return 0;
}

//--------------------------------------------------------------------------------------------------
/// The id of the state to get the geometry for
/// 
/// For non-adaptive geometries this can be ignored, as the geometry is the same for all states.
/// See CDPGeometryInfo::setHasAdaptiveGeometry for more info on adaptive geometries.
//--------------------------------------------------------------------------------------------------
cdp_int_t CDPGeometrySpec::stateId() const
{
    return -1;
}
