//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

//==================================================================================================
///
/// \class CDPLogHelper
///
/// Helper class to do bind to the raw log func provided by the host.
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// \typedef CDPLogHelper::RawLogFunc
/// The logger function provided by the CDP_FrameworkServices.
///
/// This is used to allow the plugin to log into whatever logging system the host app uses.
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
/// Create a helper by providing the rawLogFunc (as provided in the CDP_FrameworkServices)
//--------------------------------------------------------------------------------------------------
CDPLogHelper::CDPLogHelper(RawLogFunc rawLogFunc)
:   m_logFunc(rawLogFunc)
{
}

//--------------------------------------------------------------------------------------------------
/// Log error message
//--------------------------------------------------------------------------------------------------
void CDPLogHelper::logError(const std::string& message)
{
    if (m_logFunc)
    {
        m_logFunc(1, message.c_str());
    }
}

//--------------------------------------------------------------------------------------------------
/// Log warning message
//--------------------------------------------------------------------------------------------------
void CDPLogHelper::logWarning(const std::string& message)
{
    if (m_logFunc)
    {
        m_logFunc(2, message.c_str());
    }
}

//--------------------------------------------------------------------------------------------------
/// Log info message
//--------------------------------------------------------------------------------------------------
void CDPLogHelper::logInfo(const std::string& message)
{
    if (m_logFunc)
    {
        m_logFunc(3, message.c_str());
    }
}

//--------------------------------------------------------------------------------------------------
/// Log debug message
//--------------------------------------------------------------------------------------------------
void CDPLogHelper::logDebug(const std::string& message)
{
    if (m_logFunc)
    {
        m_logFunc(4, message.c_str());
    }
}

