//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#include "CDPMetaData.h"

//==================================================================================================
///
/// \class CDPElementGroupInfo
/// 
/// Contains information about one element group. 
/// 
/// This class is used for describing the parts produced by one element group if using sub-group
/// part splitting (see CDPGeometryInfo for more info on how parts are created).
/// 
/// All ids specified must be unique within the contained geometry, and parts cannot span element
/// groups, so any id used in one element group (either per group or sub-group) cannot be used in 
/// another element group.
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Set the name of the part with the given geometryGlobalPartId in this group.
/// 
/// All the unique part ids specified in the CDPElementGroup::setElementPartIds() must be specified 
/// for the associated element group with this method.
//--------------------------------------------------------------------------------------------------
void CDPElementGroupInfo::setPartName(cdp_int_t geometryGlobalPartId, const CDPString& partName)
{
}


//--------------------------------------------------------------------------------------------------
/// Set if the part with the given geometryGlobalPartId in this group is a boundary condition part or not.
///
/// All the unique part ids specified in the CDPElementGroup::setElementPartIds() must be specified
/// for the associated element group with this method.
//--------------------------------------------------------------------------------------------------
void CDPElementGroupInfo::setPartBoundaryCondition(cdp_int_t geometryGlobalPartId, bool isBoundaryCondition)
{
}


//--------------------------------------------------------------------------------------------------
/// Set the parent part id for the part with the given geometryGlobalPartId in this group.
///
/// All the unique part ids specified in the CDPElementGroup::setElementPartIds() must be specified
/// for the associated element group with this method.
//--------------------------------------------------------------------------------------------------
void CDPElementGroupInfo::setParentPart(cdp_int_t geometryGlobalPartId, cdp_int_t geometryGlobalParentPartId)
{
}


//--------------------------------------------------------------------------------------------------
/// Add additional data (key,value pairs) for a part in this geometry.
///
/// All the unique part ids specified in the CDPElementGroup::setElementPartIds() must be specified
/// for the associated element group with this method.
//--------------------------------------------------------------------------------------------------
void CDPElementGroupInfo::addPartData(cdp_int_t geometryGlobalPartId, const CDPString& key, const CDPString& value)
{
}


//==================================================================================================
///
/// \class CDPGeometryInfo
/// 
/// Contains information about one geometry.  
/// 
/// A geometry contains one or more element groups and one or more node groups. The element groups can have
/// individual node groups, or many element groups can share the same node group.
/// 
/// There are two ways to define what becomes "parts" in the host application from a data provider:
/// -# Each element group becomes one part
/// -# Each element group is split into one or more parts based on a per element assignment inside @
///    the group.
/// 
/// In the first case, each group becomes a part and the id and the name specified in the 
/// CDPGeometryInfo::addElementGroupInfo() is used for the part.
/// 
/// In the second case, you can add the element group info with the addElementGroupInfo() method 
/// (without parameters). You then add information about all the parts produced by this group
/// with the CDPElementGroupInfo::addPartInfo() method. 
/// 
/// There are two restrictions regarding part ids:
/// -# Part ids must be unique within one geometry
/// -# Part ids cannot span multiple element groups. So in the case of per element part subdivision,
///    each element group must produce unique parts.
/// 
/// See CDPDataProvider::getGeometry() for more information about how geometries are used.
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Add information about one element group. 
/// 
/// By using this method, all the elements in the group will become one part in the host and
/// it will have the given name and id.
//--------------------------------------------------------------------------------------------------
void CDPGeometryInfo::addElementGroupInfo(cdp_int_t partId, const CDPString& name)
{
}

//--------------------------------------------------------------------------------------------------
/// Add information about one element group with sub-group part splitting
///
/// Use this method if you want to provide per element part assignment within this group.
/// See description above for restrictions regarding use of part ids.
/// 
/// Use the CDPElementGroupInfo::addPartInfo() method to add meta data about each part produced
/// from this group.
/// 
/// Per element part ids are provided with the CDPElementGroup::setElementPartIds() method.
//--------------------------------------------------------------------------------------------------
CDPElementGroupInfo* CDPGeometryInfo::addElementGroupInfo()
{
    return NULL;
}

//--------------------------------------------------------------------------------------------------
/// Specify if this geometry is adaptive or not.
/// 
/// If set to true, you will be asked to provide a CDPGeometry for each state in the analysis.
/// 
/// If set to false (default), the same geometry will be used for all states and you will only
/// have to provide this once.
//--------------------------------------------------------------------------------------------------
void CDPGeometryInfo::setHasAdaptiveGeometry(bool hasAdaptiveGeo)
{
}


//==================================================================================================
///
/// \class CDPResultInfo
/// 
/// A class to provide additional information about a result. 
/// The main result information is provided when adding the result with 
/// CDPMetaData::addResultInfo().
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Set the idString of the result.
///
/// The Result ID String is a persistent identifier for the result, and will stay constant between versions of CDC.
/// This identifier will be found in the idString property of the result metadata (cee::ug::ResultInfo::idString() or cee.ug.ResultInfo.idString
///
/// If blank (default) a unique idString will be produced by combining the name and result mapping.
///
/// Make sure this is unique for each result provided by this plugin.
//--------------------------------------------------------------------------------------------------
void CDPResultInfo::setIdString(const CDPString& idString)
{
}


//--------------------------------------------------------------------------------------------------
/// Set the unit of the result.
///
/// If set, the unit will appear as text in the result info in the host app.
//--------------------------------------------------------------------------------------------------
void CDPResultInfo::setUnit(const CDPString& unit)
{
}

//--------------------------------------------------------------------------------------------------
/// Specify if the displacement result is absolute or relative displacements
/// 
/// If this is set to true, the provided displacements are absolute node position (new coordinates).
/// If set to false (default), the displacements are relative displacements offsetting the node coordinate in
/// the geometry.
//--------------------------------------------------------------------------------------------------
void CDPResultInfo::setDisplacementsContainAbsoluteNodePositions(bool containsAbsoluteNodePositions)
{
}


//--------------------------------------------------------------------------------------------------
/// Specify if the result (e.g. vector or tensor) should also produce derived results or not (default is true)
///
/// Derived results are scalars produced from vectors (x,y,z,magnitude) and vectors and scalars produced by tensors (XX stress, vonMises, 1st principal, etc)
///
/// Call setCreateDerivedResults(false) to disable this feature and only add the native result to the host meta data.
///
/// \since 3.1
//--------------------------------------------------------------------------------------------------
void CDPResultInfo::setCreateDerivedResults(bool createDerivedResults)
{
}


//==================================================================================================
///
/// \class CDPStateInfo
/// 
/// A class to provide additional information about a state. 
/// The main state information is provided when adding the state with 
/// CDPMetaData::addStateInfo().
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Specify the state group this state belongs to.
/// 
/// If using state groups, use this method to specify the group id of the parent group of this state
/// The id must refer to an existing group.
/// 
/// State groups are useful for e.g. Load Cases
//--------------------------------------------------------------------------------------------------
void CDPStateInfo::setParentStateGroupingId(cdp_int_t groupId)
{
}


//==================================================================================================
///
/// \class CDPMetaData
/// 
/// The meta data functions as a table of contents for the given analysis. It is used to inform the host application
/// about what the current simulation can provide of data. The meta data is usually used by the host
/// to populate a UI. The data provider will only be asked for data that has been reported in the meta data returned
/// in this class.
/// 
/// The meta data consists of information about:
/// - **Geometries**. The number of geometries, the name and id of each part in each geometry
/// - **States**. A list of all states with id, name, reference value etc.
/// - **State Groups**. A list of state groups (if applicable). Used to group the states in the host UI.
/// - **Sets**. A list of availble pre defined sets in the analysis.
/// - **Result Info**. A list of all the results (scalar, vector, tensor, displacements) in the analysis.
/// - **Rigid body transformations**. A bool specifying if this analysis has rigid body transformations or not.
/// - **Visibility results**. A bool specifying if the analysis has a visiblility results (per element visibility 
///     flag for all states).
/// 
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Specify information about the simulation.
///
/// This information will be available as meta data in the client app. It is only used for
/// information purposes.
//--------------------------------------------------------------------------------------------------
void CDPMetaData::setSimulationInfo(const CDPString& title, const CDPString& description, CDPSimulationType simulationType, CDPSolutionType solutionType)
{
}

//--------------------------------------------------------------------------------------------------
/// Add information about a geometry
/// 
/// All simulations need at least one geometry. Use this method to add the CDPGeometryInfo to the 
/// meta data and use the returned object to fill in information about the geometry. 
/// 
/// \sa CDPGeometryInfo 
//--------------------------------------------------------------------------------------------------
CDPGeometryInfo* CDPMetaData::addGeometryInfo()
{
    return NULL;
}

//--------------------------------------------------------------------------------------------------
/// Add information about a state
/// 
/// All simulations need at least one state. A state is a collection of geometry and results for a
/// given time step, frequency, etc. 
/// 
/// All state ids must be unique. A name should be provided, and optionally a reference value.
/// 
/// A state could be part of a state group. If so, use the returned object to specify the 
/// parent state group with the CDPStateInfo::setParentStateGroupingId() method.
//--------------------------------------------------------------------------------------------------
CDPStateInfo* CDPMetaData::addStateInfo(cdp_int_t stateId, const CDPString& name, cdp_double_t referenceValue)
{
    return NULL;
}

//--------------------------------------------------------------------------------------------------
/// Add information about a state group.
/// 
/// State groups are an optional way to group states. This information could be used by the host app
/// to build a UI where the states are organized into groups. This is purely for UI and state groups
/// will not be used when the provider is asked for geometry or results.
//--------------------------------------------------------------------------------------------------
bool CDPMetaData::addStateGroupingInfo(cdp_int_t groupId, const CDPString& name)
{
    return false;
}

//--------------------------------------------------------------------------------------------------
/// Add information of a predefined set.
/// 
/// Use this method to specify each pre defined set in the simulation. All sets needs to have a 
/// unique id. 
/// 
/// \note Currently, only Element sets are supported.
//--------------------------------------------------------------------------------------------------
bool CDPMetaData::addSetInfo(CDPSetType setType, cdp_int_t setId, const CDPString& name)
{
    return false;
}

//--------------------------------------------------------------------------------------------------
/// Add information about a result in the analysis.
/// 
/// Use this method to add information about a result in the simulation. 
/// 
/// Results are defined per item (node, element, element node, element surface) according to the result mapping.
/// Each item must be assigned a result value. A result value consists of one or more components depending on the result type.
/// 
/// The following result mappings are supported:
/// - **CDP_PER_NODE**. One result value per node
/// - **CDP_PER_ELEMENT**. One result value per element
/// - **CDP_PER_ELEMENT_NODE**. One result value per element node.
/// - **CDP_PER_ELEMENT_SURFACE**- One result value per element surface.
/// 
/// The following result types are supported:
/// - **CDP_RESTYPE_SCALAR**. Scalar result with one component per item.
/// - **CDP_RESTYPE_VECTOR**. Vector result with three components per item.
/// - **CDP_RESTYPE_DISPLACEMENT**. Displacement result with three components per item (node). Displacements can be 
///   relative (default) or absolute.
/// - **CDP_RESTYPE_STRESS_TENSOR**. Stress tensor result with six components per item.
/// - **CDP_RESTYPE_STRAIN_TENSOR**. Strain tensor result with six components per item.
/// 
/// For displacement results, we support both absolute and relative displacements (default). To specify 
/// to use absolute displacements, use the returned CDPResultInfo object and call 
/// CDPResultInfo::setDisplacementsContainAbsoluteNodePositions(true).
/// 
/// Results need to have unique ids 
//--------------------------------------------------------------------------------------------------
CDPResultInfo* CDPMetaData::addResultInfo(CDPResultType resultType, cdp_int_t resultId, const CDPString& name, CDPResultMappingType mapping)
{
    return NULL;
}

//--------------------------------------------------------------------------------------------------
/// Specify if this simulation has a rigid body transformation result.
/// 
/// If set to true, the host will query for transformation results for each state using the 
/// CDPDataProvider::getRigidBodyTransformations(). This allows for efficient movement of element groups
/// (only one 4*4 matrix per group) without having to provide displacements for each node. 
//--------------------------------------------------------------------------------------------------
void CDPMetaData::setHasRigidBodyTransformations(bool hasTransformations)
{
}

//--------------------------------------------------------------------------------------------------
/// Specify if this simulation has a per element visibility result.
///
/// If set to true, the host will query for visibility results for each state using the
/// CDPDataProvider::getVisibilityResult(). This allows for efficient removal of elements in some
/// states without having to provide a full new geometry for each state.
//--------------------------------------------------------------------------------------------------
void CDPMetaData::setHasElementVisibilityResults(bool hasVisibilityResults)
{
}
