//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#pragma once

#include "CDPBaseTypes.h"
#include "CDPDataProviderTypes.h"

class CDPString;


//==================================================================================================
//
//
//
//==================================================================================================
class CDPElementGroupInfo
{
public:
    virtual ~CDPElementGroupInfo() {};

    virtual void    setPartName(cdp_int_t geometryGlobalPartId, const CDPString& partName) = 0;
    virtual void    setPartBoundaryCondition(cdp_int_t geometryGlobalPartId, bool isBoundaryCondition) = 0;
    virtual void    setParentPart(cdp_int_t geometryGlobalPartId, cdp_int_t geometryGlobalParentPartId) = 0;

    virtual void    addPartData(cdp_int_t geometryGlobalPartId, const CDPString& key, const CDPString& value) = 0;
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CDPGeometryInfo
{
public:
    virtual ~CDPGeometryInfo() {};

    virtual void                    addElementGroupInfo(cdp_int_t partId, const CDPString& name) = 0;
    virtual CDPElementGroupInfo*    addElementGroupInfo() = 0;

    virtual void                    setHasAdaptiveGeometry(bool hasAdaptiveGeo) = 0;
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CDPResultInfo
{
public:
    virtual ~CDPResultInfo() {};

    virtual void setIdString(const CDPString& idString) = 0;
    virtual void setUnit(const CDPString& unit) = 0;
    
    virtual void setDisplacementsContainAbsoluteNodePositions(bool containsAbsoluteNodePositions) = 0;
    virtual void setCreateDerivedResults(bool createDerivedResults) = 0;
};


//==================================================================================================
//
//
//
//==================================================================================================
class CDPStateInfo
{
public:
    virtual ~CDPStateInfo() {};
    
    virtual void setParentStateGroupingId(cdp_int_t groupId) = 0;
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CDPMetaData
{
public:
    virtual ~CDPMetaData() {};
    
    virtual void                setSimulationInfo(const CDPString& title, const CDPString& description, CDPSimulationType simulationType, CDPSolutionType solutionType) = 0;
    
    virtual CDPGeometryInfo*    addGeometryInfo() = 0;
    virtual CDPStateInfo*       addStateInfo(cdp_int_t stateId, const CDPString& name, cdp_double_t referenceValue) = 0;
    virtual bool                addStateGroupingInfo(cdp_int_t groupId, const CDPString& name) = 0;

    virtual bool                addSetInfo(CDPSetType setType, cdp_int_t setId, const CDPString& name) = 0;

    virtual CDPResultInfo*      addResultInfo(CDPResultType resultType, cdp_int_t resultId, const CDPString& name, CDPResultMappingType mapping) = 0;
    virtual void                setHasRigidBodyTransformations(bool hasTransformations) = 0;
    virtual void                setHasElementVisibilityResults(bool hasVisibilityResults) = 0;
};

