//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#include "CDPResult.h"

//==================================================================================================
///
/// \class CDPResult
///
/// The result for a given geometry in a given state. 
/// 
/// This class contains an array of [groups](@ref CDPResultValueGroup). The groups are matched with the 
/// corresponding node group (for [per node](@ref CDP_PER_NODE) results) or element group (for 
/// [per element](@ref CDP_PER_ELEMENT), [per element node](@ref CDP_PER_ELEMENT_NODE) or 
/// [per element surface](@ref CDP_PER_ELEMENT_SURFACE) results).
/// 
/// The matching is done by index, so the result must have as many groups as there are element/node
/// groups in the corresponding geometry. 
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Add a result value group to this result.
/// 
/// Use this method to add the group, and populate the results for the group in the returned 
/// CDPResultValueGroup object.
//--------------------------------------------------------------------------------------------------
CDPResultValueGroup* CDPResult::addGroup()
{
    return NULL;
}


//==================================================================================================
///
/// \class CDPResultSpec
///
/// A identifier of a [result](@ref CDPResult).
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// The zero based index of the geometry to get results for
/// 
/// For the normal case of only one geometry, this will be 0.
//--------------------------------------------------------------------------------------------------
cdp_size_t CDPResultSpec::geometryIndex() const
{
    return 0;
}

//--------------------------------------------------------------------------------------------------
/// The id of the result to get.
///
/// This will always be one of the results provided in the meta data with CDPMetaData::addResultInfo
//--------------------------------------------------------------------------------------------------
cdp_int_t CDPResultSpec::resultId() const
{
    return 0;
}

//--------------------------------------------------------------------------------------------------
/// The id of the state to get results for.
/// 
/// This will always be one of the states provided in the meta data with CDPMetaData::addStateInfo
//--------------------------------------------------------------------------------------------------
cdp_int_t CDPResultSpec::stateId() const
{
    return -1;
}
