//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

//==================================================================================================
///
/// \class CDPUtils
///
/// CDP utilities
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Convert the given value to a std::string
//--------------------------------------------------------------------------------------------------
template<typename T>
std::string CDPUtils::toStdString(const T& value)
{
    std::ostringstream oss;
    oss << value;
    return oss.str();
}

//--------------------------------------------------------------------------------------------------
/// Returns a string identifying the current compiler
//--------------------------------------------------------------------------------------------------
std::string CDPUtils::compilerInfoString()
{
    std::string str = "NA";

    // MSVC
#ifdef _MSC_VER
    str = "MSVC " + toStdString(_MSC_VER);

    #ifdef _DLL
        str += ", Multithreaded DLL";
    #endif

#endif //_MSC_VER

#if defined (__APPLE__)
    str = "clang " + toStdString(__clang_major__) + "." + toStdString(__clang_minor__) + "." + toStdString(__clang_patchlevel__);
    str += ", C++: " + std::to_string(__cplusplus) + " (v:" + std::to_string(_LIBCPP_STD_VER) + ")";
    #if __has_feature(cxx_rvalue_references)
        str += ", C++11 Standard library support";
    #else
        str += ", NB! No C++11 Standard library support!";
    #endif

#elif defined (__GNUC__)
    str = "GCC " + toStdString(__GNUC__) + "." + toStdString(__GNUC_MINOR__) + "." + toStdString(__GNUC_PATCHLEVEL__);
    str += ", C++: " + std::to_string(__cplusplus) + " (libstdc++:" + std::to_string(__GLIBCXX__) + ")";

#endif //__GNUC__

#if !defined(NDEBUG)
    str += ", debug";
#endif

    return str;
}
