//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

//==================================================================================================
///
/// \class CDPVec3
/// 
/// A simple helper class for 3D vectors
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Default constructor
//--------------------------------------------------------------------------------------------------
CDPVec3::CDPVec3()
{
    m_v[0] = 0;
    m_v[1] = 0;
    m_v[2] = 0;
}

//--------------------------------------------------------------------------------------------------
/// Copy constructor
//--------------------------------------------------------------------------------------------------
CDPVec3::CDPVec3(const CDPVec3& other)
{
    m_v[0] = other.m_v[0];
    m_v[1] = other.m_v[1];
    m_v[2] = other.m_v[2];
}

//--------------------------------------------------------------------------------------------------
/// Construction from x, y, z coordinates
//--------------------------------------------------------------------------------------------------
CDPVec3::CDPVec3(double x, double y, double z)
{
    m_v[0] = x;
    m_v[1] = y;
    m_v[2] = z;
}

//--------------------------------------------------------------------------------------------------
/// Assignment operator
//--------------------------------------------------------------------------------------------------
CDPVec3& CDPVec3::operator=(const CDPVec3& rhs)
{
    m_v[0] = rhs.m_v[0];
    m_v[1] = rhs.m_v[1];
    m_v[2] = rhs.m_v[2];

    return *this;
}

//--------------------------------------------------------------------------------------------------
/// Returns vector that is this vector added with \a rhs
//--------------------------------------------------------------------------------------------------
const CDPVec3 CDPVec3::operator+(const CDPVec3& rhs) const
{
    return CDPVec3(m_v[0]+rhs.m_v[0], m_v[1]+rhs.m_v[1], m_v[2]+rhs.m_v[2]);
}

//--------------------------------------------------------------------------------------------------
/// Returns vector that is this vector subtracted with \a rhs
//--------------------------------------------------------------------------------------------------
const CDPVec3 CDPVec3::operator-(const CDPVec3& rhs) const
{
    return CDPVec3(m_v[0] - rhs.m_v[0], m_v[1] - rhs.m_v[1], m_v[2] - rhs.m_v[2]);
}

//--------------------------------------------------------------------------------------------------
/// Computes the dot product of this and \a rhs and return the result
//--------------------------------------------------------------------------------------------------
double CDPVec3::operator*(const CDPVec3& rhs) const
{
    return m_v[0]*rhs.m_v[0] + m_v[1]*rhs.m_v[1] + m_v[2]*rhs.m_v[2];
}

//--------------------------------------------------------------------------------------------------
/// Computes the cross product of this and \a rhs and return the result
//--------------------------------------------------------------------------------------------------
const CDPVec3 CDPVec3::operator^(const CDPVec3& rhs) const
{
    return CDPVec3(m_v[1]*rhs.m_v[2] - m_v[2]*rhs.m_v[1], m_v[2]*rhs.m_v[0] - m_v[0]*rhs.m_v[2], m_v[0]*rhs.m_v[1] - m_v[1]*rhs.m_v[0]); 
}

//--------------------------------------------------------------------------------------------------
/// X element of the vector
//--------------------------------------------------------------------------------------------------
const double& CDPVec3::x() const
{
    return m_v[0];
}


//--------------------------------------------------------------------------------------------------
/// Y element of the vector
//--------------------------------------------------------------------------------------------------
const double& CDPVec3::y() const
{
    return m_v[1];
}


//--------------------------------------------------------------------------------------------------
/// Z element of the vector
//--------------------------------------------------------------------------------------------------
const double& CDPVec3::z() const
{
    return m_v[2];
}


//--------------------------------------------------------------------------------------------------
/// X element of the vector
//--------------------------------------------------------------------------------------------------
double& CDPVec3::x() 
{
    return m_v[0];
}


//--------------------------------------------------------------------------------------------------
/// Y element of the vector
//--------------------------------------------------------------------------------------------------
double& CDPVec3::y() 
{
    return m_v[1];
}


//--------------------------------------------------------------------------------------------------
/// Z element of the vector
//--------------------------------------------------------------------------------------------------
double& CDPVec3::z() 
{
    return m_v[2];
}


//--------------------------------------------------------------------------------------------------
/// Sets x, y and z value
//--------------------------------------------------------------------------------------------------
void CDPVec3::set(double x, double y, double z)
{
    m_v[0] = x;
    m_v[1] = y;
    m_v[2] = z;
}


