//##################################################################################################
//
//  Ceetron Data Provider Framework
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//##################################################################################################

#pragma once

#include "CDPBaseTypes.h"
#include "CDPDataProviderTypes.h"

#include <string>
#include <vector>

class CDPDataProvider;

namespace cdphl {
class MetaData;
class Geometry;
}


//==================================================================================================
//
// 
//
//==================================================================================================
class TDProviderInterrogator
{
public:
    TDProviderInterrogator(CDPDataProvider* provider, bool verboseMode);
    ~TDProviderInterrogator();

    bool        interrogate(std::string modelKey);

private:
    void                            interrogateAllGeometries(bool showNodeCoordinates, bool showElementConnectivities);
    static void                     interrogateNodeGroupsInGeometry(const cdphl::Geometry& geo, bool showCoordinates);
    static void                     interrogateElementGroupsInGeometry(const cdphl::Geometry& geo, bool showElementConnectivities);
    void                            interrogateAllResults(bool showResultGroups, bool showResultValuesInsideGroups);
    void                            interrogateAllSets(bool showSetItems);

    static void                     outputGeometryMetaData(const cdphl::MetaData& metaData);
    static void                     outputStateMetaData(const cdphl::MetaData& metaData);
    static void                     outputResultMetaData(const cdphl::MetaData& metaData);
    static void                     outputSetMetaData(const cdphl::MetaData& metaData);

    static std::vector<cdp_int_t>   getAllStateIds(const cdphl::MetaData& metaData);
    static std::vector<cdp_int_t>   getAllResultIds(const cdphl::MetaData& metaData);

    static std::string              elementTypeToString(CDPElementType elementType);
    static std::string              resultTypeToString(CDPResultType resultType);
    static std::string              resultMappingTypeToString(CDPResultMappingType mappingType);

    static std::string              padRight(const std::string& str, size_t width);
    static std::string              padLeft(const std::string& str, size_t width);

private:
    CDPDataProvider*    m_provider;
    bool                m_verboseMode;
};
