
#include "MinimalProvider.h"

#include "CDPAll.h"




//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
MinimalProvider::MinimalProvider()
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
MinimalProvider::~MinimalProvider()
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool MinimalProvider::init(const CDPString& /*modelKey*/, const CDPInitOptions& /*initOptions*/)
{
    return true;
}

//--------------------------------------------------------------------------------------------------
///     
//--------------------------------------------------------------------------------------------------
void MinimalProvider::close()
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool MinimalProvider::getMetaData(CDPMetaData* metaData)
{
    CDPGeometryInfo* geoInfo = metaData->addGeometryInfo();
    
    // One group with per element part ids
    CDPElementGroupInfo* eltGrpInfo = geoInfo->addElementGroupInfo();
    eltGrpInfo->setPartName(1, CDPString::fromChar("Part 1"));
    eltGrpInfo->setPartName(2, CDPString::fromChar("Part 2"));

    metaData->addStateInfo(1, CDPString::fromChar("StateWithResults"), 1.0);
    metaData->addStateInfo(2, CDPString::fromChar("StateWithoutResults"), 1.0);
    
    CDPResultInfo* resInfo = metaData->addResultInfo(CDP_RESTYPE_SCALAR, 1, CDPString::fromChar("ASampleNodeResult"), CDP_PER_NODE);
    resInfo->setUnit(CDPString::fromChar("m/s"));

    return true;
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool MinimalProvider::getGeometry(const CDPGeometrySpec& spec, CDPGeometry* geometry)
{
    if (spec.geometryIndex() != 0)
    {
        return false;
    }
    
    const cdp_double_t nodeArr[] = {
        0.0, 0.0, 0.0,
        1.0, 0.0, 0.0,
        0.0, 1.0, 0.0,
        0.0, 0.0, 1.0
    };
    CDPNodeGroup* nodeGroup = geometry->addNodeGroup();
    nodeGroup->setNodes(nodeArr, sizeof(nodeArr)/(3*sizeof(cdp_double_t)));
        
    const cdp_int_t eltNodesArr[] = { 0, 1, 2,  0, 3, 1 };
    CDPElementGroup* elementGroup = geometry->addElementGroup(nodeGroup->groupIndex());
    elementGroup->addSingleTypeElements(CDP_ELT_TRIANGLES, 2, eltNodesArr, sizeof(eltNodesArr)/sizeof(cdp_int_t), NULL);
    
    const cdp_int_t partIdArr[] = { 1, 2 };
    elementGroup->setElementPartIds(partIdArr, 2);
    
    return true;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool MinimalProvider::getSet(cdp_int_t /*setId*/, CDPSet* /*set*/)
{
    return false;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool MinimalProvider::getResult(const CDPResultSpec& spec, CDPResult* result)
{
    if (spec.resultId() != 1 || spec.geometryIndex() != 0)
    {
        return false;
    }
    
    if (spec.stateId() == 1)
    {
        const cdp_double_t resArr[] = { 1.0, 2.0, 3.0, 4.0 };
        CDPResultValueGroup* group = result->addGroup();
        group->setResults(resArr, 1, sizeof(resArr)/sizeof(cdp_double_t));
    }

    return true;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool MinimalProvider::getRigidBodyTransformations(cdp_size_t /*geometryIndex*/, cdp_int_t /*stateId*/, CDPTransformations* /*transformations*/)
{
    return false;
}


//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void MinimalProvider::pollForChanges(CDPChangeNotifications* /*changeNotifications*/)
{
}
