
#include "PolyhedronProvider.h"

#include "CDPAll.h"




//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
PolyhedronProvider::PolyhedronProvider()
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
PolyhedronProvider::~PolyhedronProvider()
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool PolyhedronProvider::init(const CDPString& /*modelKey*/, const CDPInitOptions& /*initOptions*/)
{
    return true;
}

//--------------------------------------------------------------------------------------------------
///     
//--------------------------------------------------------------------------------------------------
void PolyhedronProvider::close()
{
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool PolyhedronProvider::getMetaData(CDPMetaData* metaData)
{
    CDPGeometryInfo* geoInfo = metaData->addGeometryInfo();
    
    geoInfo->addElementGroupInfo(1, CDPString::fromChar("Polyhedron Part"));

    metaData->addStateInfo(1, CDPString::fromChar("ASingleState"), 1.0);
    
    metaData->addResultInfo(CDP_RESTYPE_SCALAR, 1, CDPString::fromChar("ASampleNodeResult"), CDP_PER_NODE);

    return true;
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool PolyhedronProvider::getGeometry(const CDPGeometrySpec& spec, CDPGeometry* geometry)
{
    if (spec.geometryIndex() != 0)
    {
        return false;
    }
    
    // Add a group, producing two polyhedron elements
    const cdp_double_t nodeArr[] = {
        2.0, 0.0, 0.0,
        3.0, 0.0, 0.0,
        2.0, 1.0, 0.0,
        3.0, 1.0, 0.0,
        2.5, 1.5, 0.5,
        2.0, 0.0, 1.0,
        3.0, 0.0, 1.0,
        2.0, 1.0, 1.0,
        3.0, 1.0, 1.0
    };
        
    CDPNodeGroup* nodeGroup = geometry->addNodeGroup();
    nodeGroup->setNodes(nodeArr, sizeof(nodeArr)/(3*sizeof(cdp_double_t)));
        
    const cdp_ubyte_t numNodePerPolyArr[] = { 8, 5 };
    const cdp_ubyte_t numFacePerPolyArr[] = { 6, 5 };
        
    const cdp_ubyte_t numNodePerFaceArr[] =
    {
        // Element 0
        4, 4, 4, 4, 4, 4,
        // Element 1
        4, 3, 3, 3, 3
    };
        
    const cdp_int_t faceNodeIndiceArr[] =
    {
        // Element 0
        0, 1, 6, 5,
        8, 7, 5, 6,
        3, 8, 6, 1,
        2, 3, 1, 0,
        2, 0, 5, 7,
        2, 7, 8, 3,
            
        // Element 1
        2, 3, 8, 7,
        4, 7, 8,
        4, 8, 3,
        4, 3, 2,
        4, 2, 7,
    };
    const cdp_int_t elementNodeArr[] =
    {
        // Element 0
        0, 1, 2, 3, 5, 6, 7, 8,
        // Element 1
        2, 3, 4, 7, 8
    };
        
    CDPElementGroup* elementGroup = geometry->addElementGroup(nodeGroup->groupIndex());
    elementGroup->addSingleTypeElements(CDP_ELT_POLYHEDRONS, 2, elementNodeArr, 13, NULL);
    elementGroup->addPolyhedronData(numNodePerPolyArr, numFacePerPolyArr, 2,
                                    numNodePerFaceArr, 11,
                                    faceNodeIndiceArr, 40);
    
    return true;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool PolyhedronProvider::getSet(cdp_int_t /*setId*/, CDPSet* /*set*/)
{
    return false;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool PolyhedronProvider::getResult(const CDPResultSpec& spec, CDPResult* result)
{
    if (spec.resultId() != 1 || spec.geometryIndex() != 0)
    {
        return false;
    }
    
    const cdp_double_t resArr[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0 };
    CDPResultValueGroup* group = result->addGroup();
    group->setResults(resArr, 1, sizeof(resArr)/sizeof(cdp_double_t));

    return true;
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool PolyhedronProvider::getRigidBodyTransformations(cdp_size_t /*geometryIndex*/, cdp_int_t /*stateId*/, CDPTransformations* /*transformations*/)
{
    return false;
}


//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void PolyhedronProvider::pollForChanges(CDPChangeNotifications* /*changeNotifications*/)
{
}
